from sequana_pipetools.options import *
from easydev import AttrDict
import argparse


def test_misc():

    import sys
    sys.argv.append("--version")
    from sequana_pipetools.options import init_pipeline
    try:
        init_pipeline("rnaseq")
    except:
        pass
    sys.argv.remove("--version")
    sys.argv.append("--deps")
    try:
        init_pipeline("rnaseq")
    except:
        pass




def test_cutadapt_options():

    p = argparse.ArgumentParser()
    so = CutadaptOptions()
    so.add_options(p)

    # test the adapter choice
    for this in ["universal", "PCRFree", "none"]:
        options = {
            "cutadapt_adapter_choice": this,
            "cutadapt_design_file": None,
            "cutadapt_fwd": None,
            "cutadapt_rev": None,
            "skip_cutadapt": False,
        }
        #p.parse_args([])
        options = AttrDict(**options)
        so.check_options(options)

    # This one fails because a design and fwd/rev are provided together
    for this in ["universal", "PCRFree", "none"]:
        options = {
            "cutadapt_adapter_choice": this,
            "cutadapt_design_file": "whatever",
            "cutadapt_fwd": "ACGT",
            "cutadapt_rev": "ACGT",
            "skip_cutadapt": False,
        }
        #p.parse_args([])
        options = AttrDict(**options)
        try:
            so.check_options(options)
            assert False
        except: assert True

    # This one fails because a design but not adapter choice
    options = {
        "cutadapt_adapter_choice": None,
        "cutadapt_design_file": "whatever",
        "cutadapt_fwd": None,
        "cutadapt_rev": None,
        "skip_cutadapt": False,
    }
    options = AttrDict(**options)
    try:
        so.check_options(options)
        assert False
    except: assert True

    # This one fails because adapter file is not correct
    options = {
        "cutadapt_adapter_choice": "PCRFree",
        "cutadapt_design_file": "whatever",
        "cutadapt_fwd": None,
        "cutadapt_rev": None,
        "skip_cutadapt": False,
    }
    options = AttrDict(**options)
    try:
        so.check_options(options)
        assert False
    except: assert True


    # wrong combo (missing adapter choice)
    options = {
        "cutadapt_adapter_choice": None,
        "cutadapt_design_file": None,
        "cutadapt_fwd": None,
        "cutadapt_rev": None,
        "skip_cutadapt": False,
    }
    options = AttrDict(**options)
    try:
        so.check_options(options)
        assert False
    except: assert True

    # wrong quality (missing adapter choice)
    try:
        p.parse_args(["--cutadapt-quality", "-1"])
        assert False
    except: assert True
    p.parse_args(["--cutadapt-quality", "10"])

    # test for a valid design and adapter choice but also fwd/rev provided
    # whereas, we cannot do anything with this combo
    options = {
        "cutadapt_adapter_choice": "TruSeq",
        "cutadapt_design_file": None,
        "cutadapt_fwd": "ACGT",  # dummy values
        "cutadapt_rev": "CGTA",  # dummy values
        "skip_cutadapt": False,
    }
    options = AttrDict(**options)
    try:
        so.check_options(options)
        assert False
    except: assert True


def test_snakemake_options():
    from sequana_pipetools.options import SnakemakeOptions
    p = argparse.ArgumentParser()
    so = SnakemakeOptions()
    so.add_options(p)
    p.parse_args([])


def test_krakenl_options():
    from sequana_pipetools.options import KrakenOptions
    p = argparse.ArgumentParser()
    so = KrakenOptions()
    so.add_options(p)
    p.parse_args([])


def test_slurm_options():
    from sequana_pipetools.options import SlurmOptions
    p = argparse.ArgumentParser()
    so = SlurmOptions()
    so.add_options(p)
    p.parse_args([])


def test_input_options():
    from sequana_pipetools.options import InputOptions
    p = argparse.ArgumentParser()
    so = InputOptions()
    so.add_options(p)
    p.parse_args([])


def test_general_options():
    from sequana_pipetools.options import GeneralOptions
    p = argparse.ArgumentParser()
    so = GeneralOptions()
    so.add_options(p)
    p.parse_args([])



# -*- coding: utf-8 -*-
#
#  This file is part of Sequana_pipetools software (Sequana project)
#
#  Copyright (c) 2020 - Sequana Development Team
#
#  File author(s):
#      Thomas Cokelaer <thomas.cokelaer@pasteur.fr>
#
#  Distributed under the terms of the 3-clause BSD license.
#  The full license is in the LICENSE file, distributed with this software.
#
#  website: https://github.com/sequana/sequana
#  documentation: http://sequana.readthedocs.io
#
##############################################################################
import sys
from sequana_pipetools.misc import Colors, print_version, print_newest_version


__all__ = ["GeneralOptions", "SlurmOptions", "SnakemakeOptions",
    "CutadaptOptions", "KrakenOptions", "InputOptions", ]


def init_pipeline(NAME):
    print('deprecated, please use before_pipeline instead')
    before_pipeline(NAME)

def before_pipeline(NAME):
    """A function to provide --version and --deps for all pipelines

    """

    if "--version" in sys.argv:
        print_version(NAME)
        if "--any-update" in sys.argv:
            print_newest_version(["sequana", "sequana-pipetools", "sequana-" + NAME])
        sys.exit(0)

    if "--deps" in sys.argv:
        # Means than sequana must be installed, which we assume if someone uses
        # a pipeline. so must be here and not global import
        from sequana.snaketools import Module
        module = Module(NAME)
        with open(module.requirements, "r") as fin:
            data = fin.read()
        print("Those software will be required for the pipeline to work correctly:\n{}".format(data))
        sys.exit(0)

    # This slow down all --help, so only when using --version
    #print_newest_version(["sequana", "sequana-pipetools", "sequana-" + NAME])


class GeneralOptions():
    def __init__(self):
        pass

    def add_options(self, parser):
        parser.add_argument(
            "--run-mode",
            dest="run_mode",
            default=None,
            choices=['local', 'slurm'],
            help="""run_mode can be either 'local' or 'slurm'. Use local to
                run the pipeline locally, otherwise use 'slurm' to run on a
                cluster with SLURM scheduler. Other clusters are not maintained.
                However, you can set to slurm and change the output shell script
                to fulfill your needs. If unset, sequana searches for the sbatch
                and srun commands. If found, this is set automatically to
                'slurm', otherwise to 'local'.
                """)

        parser.add_argument("--version", action="store_true",
            help="Print the version and quit")
        parser.add_argument("--deps", action="store_true",
            help="Show the known dependencies of the pipeline")
        parser.add_argument("--level", dest="level", default="INFO",
            choices=['INFO', 'DEBUG', 'WARNING', 'ERROR', 'CRITICAL'],
            help="logging level in INFO, DEBUG, WARNING, ERROR, CRITICAL")
        parser.add_argument("--from-project", dest="from_project", default=None,
            help="""You can initiate a new analysis run from an existing project.
                In theory, sequana project have a hidden .sequana directory, 
                which can be used as input. The name of the run directory itself 
                should suffice (if .sequana is found inside). From there, 
                the config file and the pipeline files are copied in your new 
                working directory""")


def guess_scheduler():
    """Guesses whether we are on a SLURM cluster or not.

    If not, we assume a local run is expected.
    """
    from easydev import cmd_exists
    if cmd_exists("sbatch") and cmd_exists("srun"):
        return 'slurm'
    else:
        return 'local'



class SnakemakeOptions():
    def __init__(self, group_name="snakemake", working_directory="analysis"):
        self.group_name = group_name
        self.workdir = working_directory

    def _default_jobs(self):
        if guess_scheduler() == "slurm":
            return 40
        else:
            return 4

    def add_options(self, parser):
        group = parser.add_argument_group(self.group_name)

        group.add_argument(
            "--jobs",
            dest="jobs",
            default=self._default_jobs(),
            help="""Number of jobs to run at the same time (default 4 on a local
                computer, 40 on a SLURM scheduler). This is the --jobs options
                of Snakemake"""
        )
        group.add_argument(
            "--working-directory",
            dest="workdir",
            default=self.workdir,
            help="""where to save the pipeline and its configuration file and
            where the analyse can be run"""
        )
        group.add_argument(
            "--force",
            dest="force",
            action="store_true",
            default=False,
            help="""If the working directory exists, proceed anyway."""
        )




class InputOptions():
    def __init__(self, group_name="data", input_directory=".",
                 input_pattern="*fastq.gz", add_input_readtag=True):
        """

        By default, single-end data sets. If paired, set is_paired to True
        If so, the add_input_readtag must be set
        """
        self.group_name = group_name
        self.input_directory = input_directory
        self.input_pattern = input_pattern
        self.add_input_readtag = add_input_readtag

    def add_options(self, parser):
        self.group = parser.add_argument_group(self.group_name)
        self.group.add_argument(
             "--input-directory",
             dest="input_directory",
             default=self.input_directory,
             #required=True,
             help="""Where to find the FastQ files""",
        )
        self.group.add_argument(
            "--input-pattern",
            dest="input_pattern",
            default=self.input_pattern,
            help="pattern for the input FastQ files ",
        )

        if self.add_input_readtag:
            self.group.add_argument(
                "--input-readtag",
                dest="input_readtag",
                default="_R[12]_",
                help="""pattern for the paired/single end FastQ. If your files are
                tagged with _R1_ or _R2_, please set this value to '_R[12]_'. If your
                files are tagged with  _1 and _2, you must change this readtag
                accordingly to '_[12]'. This option is used only if
                --paired-data is used""",
            )


class KrakenOptions():
    def __init__(self, group_name="section_kraken"):
        self.group_name = group_name

    def add_options(self, parser):
        group = parser.add_argument_group(self.group_name)

        group.add_argument("--skip-kraken", action="store_true",
            default=False,
            help="""If provided, kraken taxonomy is performed. A database must be
                provided (see below). """)

        group.add_argument("--kraken-databases", dest="kraken_databases", type=str,
            nargs="+", default=[],
            help="""Path to a valid set of Kraken database(s).
                If you do not have any, please see https://sequana.readthedocs.io
                or use sequana_taxonomy --download option.
                You may use several, in which case, an iterative taxonomy is
                performed as explained in online sequana documentation""")


class CutadaptOptions():
    description = """
    This section allows you to trim bases (--cutadapt-quality) with poor
    quality and/or remove adapters.

    To remove adapters, several options are possible:

    (1) you may use an experimental design file (--cutadapt-design-file),
    in which case the type of adapters is also required with the option
    --cutadapt-adapter-choice.
    (2) specify the name of the adapters(--cutadapt-adapter-choice)
    e.g. PCRFree. You may specify "universal" to remove universal
    adapters only.
    (3) provide the adapters directly as a string (or a file) using
    --cutadapt-fwd (AND --cutadapt-rev" for paired-end data).

    If you set the --cutadapt-adapter-choice to 'none', fwd and reverse
    adapters are set to XXXX (see cutadapt documentation).

    """

    adapters_choice = ["none", "universal", "Nextera", "Rubicon", "PCRFree",
        "TruSeq", "SMARTer", "Small"]

    def __init__(self, group_name="section_cutadapt"):
        self.group_name = group_name

    def add_options(self, parser):

        group = parser.add_argument_group(self.group_name, self.description)

        group.add_argument("--skip-cutadapt", action="store_true",
            default=False,
             help="If provided, fastq cleaning and trimming will be skipped")

        group.add_argument("--cutadapt-fwd", dest="cutadapt_fwd",
            default="",
            help="""Provide a adapter as a string of stored in a
                FASTA file. If the file exists, we will store it as expected
                with a preceeding prefix 'file:'""")

        group.add_argument("--cutadapt-rev", dest="cutadapt_rev",
            default="",
            help="""Provide a adapter as a string of stored in a
                FASTA file. If the file exists, we will store it as expected
                with a preceeding prefix 'file:'""")

        def quality(x):
            x = int(x)
            if x < 0:
                raise argparse.ArgumentTypeError("quality must be positive")
            return x

        group.add_argument("--cutadapt-quality", dest="cutadapt_quality",
            default=30, type=quality,
            help="""0  means no trimming, 30 means keep bases with quality
                above 30""")

        group.add_argument("--cutadapt-tool-choice", dest="cutadapt_tool_choice",
            default="cutadapt", choices=["cutadapt", "atropos"],
            help="Select the prefered tool. Default is cutadapt")

        group.add_argument("--cutadapt-adapter-choice",
            dest="cutadapt_adapter_choice",
            default=None, choices=self.adapters_choice,
            help="""Select the adapters used that may possibly still be
                present in the sequences""")

        group.add_argument("--cutadapt-design-file", dest="cutadapt_design_file",
            default=None,
            help="A valid CSV file with mapping of adapter index and sample name")

        group.add_argument("--cutadapt-mode", dest="cutadapt_mode",
            default="b", choices=["g", "a", "b"],
            help="""Mode used to remove adapters. g for 5', a for 3', b for both
                5'/3' as defined in cutadapt documentation""")

        group.add_argument("--cutadapt-options", dest="cutadapt_options",
            default=" -O 6 --trim-n",
            help="""additional options understood by cutadapt""")

    def check_options(self, options):
        """
        """
        design = options.cutadapt_design_file
        adapter_choice = options.cutadapt_adapter_choice
        adapter_fwd = options.cutadapt_fwd
        adapter_rev = options.cutadapt_rev

        if design:
            if adapter_fwd or adapter_rev:
                logger.critical(
                    "When using --cutadapt-design-file, one must not"
                    " set the forward/reverse adapters with --cutadapt-fwd"
                    " and/or --cutadapt-rev\n\n" + self.description)
                sys.exit(1)

            # otherwise, we just check the format but we need the adapter choice
            if options.cutadapt_adapter_choice in [None, 'none']:
                logger.critical(
                    "When using --cutadapt-design-file, you must also"
                    " provide the type of adapters using --cutadapt-adapter-choice"
                    " (set to one of %s )" % self.adapters_choice)
                sys.exit(1)

            from sequana import FindAdaptersFromDesign
            fa = FindAdaptersFromDesign(design, options.cutadapt_adapter_choice)
            try:
                fa.check()
            except:
                logger.critical("Your design file contains indexes not found "
                    "in the list of adapters from {}".format(options.cutadapt_adapter_choice))
                sys.exit(1)

        # No design provided here below
        # do we need to remove adapters at all ?
        elif options.cutadapt_adapter_choice == "none":
            options.cutadapt_adapter_choice = None
            options.cutadapt_fwd = "XXXX"
            options.cutadapt_rev = "XXXX"
        # or just the universal ones ?
        elif options.cutadapt_adapter_choice == "universal":
            options.cutadapt_fwd = "GATCGGAAGAGCACACGTCTGAACTCCAGTCACCGATGTATCTCGTATGCCGTCTTCTGC"
            options.cutadapt_rev = "TCTAGCCTTCTCGCAGCACATCCCTTTCTCACATCTAGAGCCACCAGCGGCATAGTAA"
        # or do we have a string or files provided for the fwd/rev ?
        elif options.cutadapt_adapter_choice is None:
            if options.cutadapt_fwd:
                # Could be a string or a file. If a file, check the format
                if os.path.exists(options.cutadapt_fwd):
                    AdapterReader(options.cutadapt_fwd)
                    options.cutadapt_fwd = "file:{}".format(
                        os.path.abspath(options.cutadapt_fwd))
            if options.cutadapt_rev:
                # Could be a string or a file. If a file, check the format
                if os.path.exists(options.cutadapt_rev):
                    AdapterReader(options.cutadapt_rev)
                    options.cutadapt_rev = "file:{}".format(
                        os.path.abspath(options.cutadapt_rev))
        elif options.cutadapt_adapter_choice:
            # nothing to do, the cutadapt rules from sequana will use 
            # the adapter_choice, and fill the fwd/rev automatically
            pass





class SlurmOptions():
    def __init__(self, group_name="slurm", memory=4000, queue="common", cores=4):
        """

        ::

            class Options(argparse.ArgumentParser, SlurmOptions):
                def __init__(self, prog="whatever")
                    super(Options, self).__init__(usage="todo",
                        prog="whatever",description=""
                    self.add_argument(...)
                    ...
                    self.add_slurm_options()

        """
        self.group_name = group_name
        self.memory = memory
        self.cores = cores
        self.queue = queue

    def add_options(self, parser):
        group = parser.add_argument_group(self.group_name)
        group.add_argument(
            "--slurm-cores-per-job",
            dest="slurm_cores_per_job",
            default=self.cores,
            help="""Number of cores/jobs to be used at the same time.
            Ignored and replaced if a cluster_config.yaml file is part
            of your pipeline (e.g. rnaseq)""",
        )
        group.add_argument(
            "--slurm-queue",
            dest="slurm_queue",
            default=self.queue,
            help="SLURM queue to be used (biomics)",
        )
        group.add_argument(
            "--slurm-memory",
            dest="slurm_memory",
            default=self.memory,
            help="""memory in Mb (default 4000; stands for 4000 Mbytes).
            Ignored and replaced if a cluster_config.yaml file is part
            of your pipeline (e.g. rnaseq)""",
        )





import sys
import os
import argparse
import shutil
from subprocess import check_call
import subprocess

__all__ = ['Complete']


class Options(argparse.ArgumentParser):
    def __init__(self, prog="sequana_completion"):
        usage = """
    sequana_completion --name rnaseq
    sequana_completion --name all
    """

        super(Options, self).__init__(usage=usage, prog=prog,
            description="""This tool creates completion script for sequana
pipelines. Each pipeline has its own in .config/sequana/pipelines that you can
source at your convenience""",
            formatter_class=argparse.ArgumentDefaultsHelpFormatter)

        self.add_argument("--force", action="store_true",
            help="""overwrite files in sequana config pipeline directory""")
        self.add_argument("--name",  type=str,
            help="""Name of a pipelines for which you wish to
            create the completion file. Set to a valid name ror to create all
scripts, use --name all """)


class Complete():

    # For the -o default, this was an issue with compgen removing the slashes on
    # directrories . Solution was found here: 
    # https://stackoverflow.com/questions/12933362/getting-compgen-to-include-slashes-on-directories-when-looking-for-files
    # Before using this option, a second directory could not be completed e.g.
    # in --databases, only the first argument could be completed, which was
    # really annoying. 

    # KEEP '#version:' on first line as it it since it is used in
    # sequana/pipeline_common.py right now
    setup = """#version: {version}
function _mycomplete_{pipeline_name}()
{{
    local cur prev opts
    COMPREPLY=()
    cur="${{COMP_WORDS[COMP_CWORD]}}"
    prev="${{COMP_WORDS[COMP_CWORD-1]}}"
    opts="{options}"
    case "${{prev}}" in
    """

    teardown = """
        #;;
    esac
    #if [[ ${{cur}} == -* ]] ; then
        COMPREPLY=( $(compgen -W "${{opts}}" -- ${{cur}}) )
        return 0
    #fi

}}
#complete -d -X '.[^./]*' -F _mycomplete_ sequana_{pipeline_name}
complete -o  default -F _mycomplete_{pipeline_name} sequana_{pipeline_name}
    """

    def __init__(self, pipeline_name):
        self._set_pipeline_name(pipeline_name)

    def _get_pipeline_name(self):
        return self._pipeline_name

    def _set_pipeline_name(self, name):
        self._pipeline_name = name
        self._init_config_file()
        self._init_version()
    pipeline_name = property(_get_pipeline_name, _set_pipeline_name)

    def save_completion_script(self):
        config_path = self.config_path
        pipeline_name = self.pipeline_name

        output_filename = f"{config_path}/{pipeline_name}.sh"

        arguments = self.get_arguments()

        with open(output_filename, "w") as fout:
            fout.write(self.setup.format(version=self.pipeline_version,
                                        pipeline_name=pipeline_name,
                                         options=" ".join(arguments)))
            for action in self._actions:
                option_name = action.option_strings[0]
                if action.choices:
                    option_choices = " ".join(action.choices )
                    fout.write(self.set_option_with_choice(option_name, option_choices))
                elif option_name.endswith("-directory"):
                    fout.write(self.set_option_directory(option_name))
                elif option_name.endswith("-file"):
                    fout.write(self.set_option_file(option_name))
                elif option_name in ["--databases", "--from-project"]:
                    fout.write(self.set_option_directory(option_name))
            fout.write(self.teardown.format(pipeline_name=self.pipeline_name))

    def _init_config_file(self):
        # do not import sequana, save time, let us use easydev
        from easydev import CustomConfig
        configuration = CustomConfig("sequana", verbose=False)
        sequana_config_path = configuration.user_config_dir
        path = sequana_config_path + os.sep + "pipelines"
        if os.path.exists(path):
            pass
        else:
            os.mkdir(path)
        self.config_path = path
        return path

    def _init_version(self):
        import importlib
        pname = self.pipeline_name
        mod = importlib.import_module('sequana_pipelines.{}'.format(pname))
        mod = importlib.import_module('sequana_pipelines.{}.main'.format(pname))
        module = sys.modules['sequana_pipelines.{}'.format(self.pipeline_name)]
        version = module.version
        self.pipeline_version = version

    def get_arguments(self):
        import importlib
        pname = self.pipeline_name
        mod = importlib.import_module('sequana_pipelines.{}'.format(pname))
        mod = importlib.import_module('sequana_pipelines.{}.main'.format(pname))
        module = sys.modules['sequana_pipelines.{}'.format(self.pipeline_name)]

        main = module.__getattribute__('main')
        opt = main.Options()
        to_exclude = ["-h", "--help"]
        self._actions = [a for a in opt._actions if a.option_strings[0] not in
to_exclude]
        arguments = [a.option_strings[0] for a in self._actions]
        return arguments

    def set_option_with_choice(self, option_name, option_choices):
        data =  f"""
            {option_name})
                local choices="{option_choices}"
                COMPREPLY=( $(compgen -W "${{choices}}" -- ${{cur}}) )
                return 0
                ;;"""
        return data

    def set_option_directory(self, option_name):
        data = f"""
            {option_name})
                xpat=".[!.]*"
                COMPREPLY=( $(compgen -X "${{xpat}}" -d ${{cur}}) )
                return 0
                ;;"""
        return data

    def set_option_file(self, option_name):
        data = f"""
            {option_name})
                #xpat=".[!.]*"
                #COMPREPLY=( $(compgen -X "${{xpat}}" -d ${{cur}}) )
                COMPREPLY=( $(compgen  -f ${{cur}}) )
                return 0
                ;;"""
        return data





def main(args=None):

    if args is None:
        args = sys.argv[:]

    user_options = Options()

    # If --help or no options provided, show the help
    if len(args) == 1:
        user_options.parse_args(["prog", "--help"])
    else:
       options = user_options.parse_args(args[1:])

    names = []
    if options.name == "all":
        import pkgutil
        import sequana_pipelines
        for ff, module_name, valid in pkgutil.iter_modules(sequana_pipelines.__path__):
            names.append(module_name)
    else:
        names = [options.name]

    if options.force is False:
        msg = ("This will replace files in ./config/sequana/pipelines. "
            "Do you want to proceed y/n: ")
        choice = input(msg)
    else:
        choice = "y"

    if choice == "y":
        print("Please source the files using:: \n")
        for name in names:
            c = Complete(name)
            c.save_completion_script()
            print("source ~/.config/sequana/pipelines/{}.sh".format(name))
        print("\nto activate the completion")
    else:
        print("Stopping creation of completion scripts")


if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
#
#  This file is part of Sequana_pipetools software (Sequana Projetc)
#
#  Copyright (c) 2020 - Sequana Development Team
#
#  File author(s):
#      Thomas Cokelaer <thomas.cokelaer@pasteur.fr>
#
#  Distributed under the terms of the 3-clause BSD license.
#  The full license is in the LICENSE file, distributed with this software.
#
#  website: https://github.com/sequana/sequana
#  documentation: http://sequana.readthedocs.io
#
##############################################################################
import sys
import os

__all__ = ["Colors", "print_version", "error"]


class Colors:
    """

    ::

        color = Colors()
        print(color.failed("msg"))

    """
    PURPLE = "\033[95m"
    BLUE = "\033[94m"
    GREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    ENDC = "\033[0m"
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"

    def failed(self, msg):
        return self.FAIL + msg + self.ENDC

    def bold(self, msg):
        return self.BOLD + msg + self.ENDC

    def purple(self, msg):
        return self.PURPLE + msg + self.ENDC

    def underlined(self, msg):
        return self.UNDERLINE + msg + self.ENDC

    def fail(self, msg):
        return self.FAIL + msg + self.ENDC

    def error(self, msg):
        return self.FAIL + msg + self.ENDC

    def warning(self, msg):
        return self.WARNING + msg + self.ENDC

    def green(self, msg):
        return self.GREEN + msg + self.ENDC

    def blue(self, msg):
        return self.BLUE + msg + self.ENDC



def error(msg, pipeline):
    color = Colors()
    print(color.error("ERROR [sequana.{}]::".format(pipeline) +  msg), flush=True)
    sys.exit(1)


def print_version(name):
    try:
        import pkg_resources
        version = pkg_resources.require("sequana")[0].version
        print("Sequana version used: {}".format(version))
    except Exception as err:
        print(err)
        print("Sequana version used: ?".format(name))

    try:
        import pkg_resources
        version = pkg_resources.require("sequana_pipetools")[0].version
        print("Sequana_pipetools version used: {}".format(version))
    except Exception as err:
        print(err)
        print("Sequana_pipetools version used: ?".format(name))

    try:
        ver = pkg_resources.require("sequana_{}".format(name))[0].version
        print("pipeline sequana_{} version used: {}".format(name, ver))
    except Exception as err:
        print(err)
        print("pipeline sequana_{} version used: ?".format(name))
        sys.exit(1)
    print(Colors().purple("\nHow to help ?\n- Please, consider citing us (see sequana.readthedocs.io)".format(version)))
    print(Colors().purple("- Contribute to the code or documentation"))
    print(Colors().purple("- Fill issues on https://github.com/sequana/sequana/issues/new/choose"))
    print(Colors().purple("- Star us https://github.com/sequana/sequana/stargazers"))


def print_newest_version(name=None):
    # This is slow
    color = Colors()
    import subprocess
    ret = subprocess.run(["pip", "list", "--outdated"], stdout=subprocess.PIPE)

    if name:
        if isinstance(name, str):
            names = [name]
        elif isinstance(name, list):
            names = name
    else:
        names = None

    for line in ret.stdout.strip().decode().split("\n"):

        if names:
            if line.split()[0].strip() in names:
                pkg = line.split()[0]
                local_version = line.split()[1]
                new_version = line.split()[2]
                print(color.warning(
                    "A newest version ({}) is available for {}. You have {}".format(
                    new_version, pkg, local_version)))
        elif line.split()[0].startswith("sequana"):
            pkg = line.split()[0]
            local_version = line.split()[1]
            new_version = line.split()[2]
            print(color.warning(
                "A newest version ({}) is available for {}. You have {}".format(
                new_version, pkg, local_version)))
